<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
  
  <xsl:output indent="yes" method="xml"
    doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
  
  <xsl:template match="/">
    <xsl:result-document href="index.html">
      <xsl:apply-templates select="book"/>
    </xsl:result-document>
    
    <xsl:for-each select="book/chapter">
      <xsl:result-document href="{generate-id(.)}.html">
        <xsl:apply-templates select="."/>
      </xsl:result-document>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="book">
    <html>
      <head>
        <title><xsl:value-of select="title"/></title>
      </head>
      <body>
        <h1><xsl:value-of select="title"/></h1>
        <h2>Table of contents</h2>
        <ul>
          <xsl:for-each select="chapter">
            <li>
              <a href="{generate-id(.)}.html"><xsl:value-of select="title"/></a>
            </li>
          </xsl:for-each>
        </ul>
      </body>
    </html>
  </xsl:template>
  
  <xsl:template match="chapter">
    <html>
      <head>
        <title><xsl:value-of select="title"/></title>
      </head>
      <body>
        <h2 id="{generate-id()}">
          <a href="index.html"><xsl:value-of select="title"/></a>
        </h2>
        <xsl:apply-templates select="para|itemizedlist|table"/>
      </body>
    </html>
  </xsl:template>
  
  <xsl:template match="para">
    <p id="{generate-id(.)}"><xsl:apply-templates select="text()|*" /></p>
  </xsl:template>
  
  <xsl:template match="emphasis">
    <em><xsl:value-of select="."/></em>
  </xsl:template>
  
  <xsl:template match="link">
    <xsl:variable name="reftargetNode" select="id(@linkend)"/>
    <xsl:variable name="reftargetParentChapter"
      select="$reftargetNode/ancestor-or-self::chapter"/>
    
    <a href="{generate-id($reftargetParentChapter)}.html#{
      generate-id($reftargetNode)}">
      <xsl:value-of select="."/>
    </a>
  </xsl:template>
  
  <xsl:template match="itemizedlist">
    <ul>
      <xsl:apply-templates select="listitem"/>
    </ul>
  </xsl:template>
  
  <xsl:template match="listitem">
    <li>
      <xsl:apply-templates select="*"/>
    </li>
  </xsl:template>
  
  <xsl:template match="table">
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="*">
    <xsl:message>
      <xsl:text>No template defined for element '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>'</xsl:text>
    </xsl:message>
  </xsl:template>
  
</xsl:stylesheet>