package de.hdm_stuttgart.sw1.klausur.test;

import org.junit.Assert;
import org.junit.Test;
import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;

import org.junit.Rule;
import org.junit.rules.DisableOnDebug;
import org.junit.rules.TestRule;
import org.junit.rules.Timeout;

import de.hdm_stuttgart.mi.exam.MarkingTest;
import de.hdm_stuttgart.sw1.klausur.string.StringHelper;

@SuppressWarnings({"static-method", "javadoc"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class TestFilterSortiere  extends MarkingTest {

        @Rule // Kill test if lasting longer than 1000 milliseconds unless in debugging mode
        public final TestRule timeout = new DisableOnDebug(Timeout.millis(1000));
    
	@Test
	public void nullArray() {

		final String[] result = StringHelper.filterUndSortiereDuplikate(null);
		assertArrayEquals(new String[0], result);
	}

	@Test
	public void einzelnerNullWert() {

		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{null});
		assertArrayEquals(new String[0], result);
	}

	@Test
	public void mehrereNullWerte() {

		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{null});
		assertArrayEquals(new String[0], result);
	}

	@Test
	public void einzelnNotNull() {

		final String testWert = "Hans";
		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{testWert});
		assertArrayEquals(new String[]{testWert}, result);
	}

	@Test
	public void mehrfachIdentischNotNull() {

		final String testWert = "Hans";
		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{testWert, testWert, testWert});
		assertArrayEquals(new String[]{testWert}, result);
	}

	@Test
	public void mehrfachIdentischNull() {

		final String testWert = "Hans";
		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{testWert, null, testWert});

		assertArrayEquals(new String[]{testWert}, result);
	}

	@Test
	public void verschieden() {

		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{"eins", "Zwei", "Drei"})
				, expected = new String[]{"Drei", "Zwei", "eins"};
		assertArrayEquals(expected, result);

	}

	@Test
	public void duplikate() {

		final String[] result = StringHelper.filterUndSortiereDuplikate(new String[]{"eins", "Zwei", "Drei"})
				, expected = new String[]{"Drei", "Zwei", "eins"};
		assertArrayEquals(expected, result);

	}

	/**
	 * Hilfsmethode zum Test der Gleichheit zweier String arrays.
	 * 
	 * @param expected Erwartetes Array
	 * @param result Erhaltenes Array
	 */
	public static void assertArrayEquals(final String[] expected, final String[] result) {
		Assert.assertEquals("String Länge verschieden: " + expected.length + " != " + result.length, expected.length, result.length);
		for (int i = 0; i < expected.length; i++) {
			Assert.assertEquals("Array Komponenten sind verschieden: " + expected[i] + " != " + result[i], expected[i], result[i]);
		}
	}


}
