package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Jährliche Tarifberechnung für Gasverbraucher. Die Angabe von Gasmengen erfolgt
 *    in Kilowattstunden (kwh).</p>
 * 
 * <p>Es werden zwei Tarifmodelle angeboten:</p>
 * 
 * <dl>
 * 
 *   <dt>Kleinverbraucher:</dt>
 *   <dd>
 *     <p>Der jährliche {@link #KLEINVERBRAUCHER_GRUNDPREIS}
 *        ist niedriger als bei Großverbrauchern. Dafür ist der pro
 *        verbrauchter Kilowattstunde berechnete 
 *        {@link #KLEINVERBRAUCHER_ARBEITSPREIS} höher.</p>
 *       
 *     <p>Beispiel: Für einen Jahresverbrauch von 5000 Kilowattstunden ergeben sich
 *        die Gesamtkosten zu 50,00€ Grundgebühr + 5000 * 0,12€ = 650,00€</p>
 *   </dd>
 * 
 *   <dt>Großverbraucher:</dt>
 *   <dd>
 *     <p>Der {@link #GROSSVERBRAUCHER_GRUNDPREIS} ist höher als bei Kleinverbrauchern.
 *       Dafür ist der pro verbrauchter Kilowattstunde berechnete
 *       {@link #GROSSVERBRAUCHER_ARBEITSPREIS} niedriger.</p>
 *        
 *     <p>Beispiel: Für einen Jahresverbrauch von 30000 Kilowattstunden ergeben sich
 *         die Gesamtkosten zu 400,00€ Grundgebühr + 30000 * 0,04€ = 1600,00€</p>
 *   </dd>
 * </dl>
 *
 */
public class Gaspreise {
   
   /**
    * Jährlicher Grundpreis in € für Kleinverbraucher.
    */
   static public final double KLEINVERBRAUCHER_GRUNDPREIS = 50.00;
   
   /**
    * Kosten in € pro Kilowattstunde für Kleinverbraucher.
    */
   static public final double KLEINVERBRAUCHER_ARBEITSPREIS = 0.12;
   
   /**
    * Jährlicher Grundpreis in € für Großverbraucher.
    */
   static public final double GROSSVERBRAUCHER_GRUNDPREIS = 400.00;

   /**
    * Kosten in € pro Kilowattstunde Großverbraucher.
    */
   static public final double GROSSVERBRAUCHER_ARBEITSPREIS = .04;


   /**
    * <p>Gesamtkosten pro Jahr auf Basis des Tarifs für Kleinverbraucher.</p>
    * 
    * @param kilowattstunden Jahresgasverbrauch
    * @return Zu zahlender Gesamtbetrag.
    */
   static public double getKostenKleinverbrauch(int kilowattstunden) {
      return KLEINVERBRAUCHER_GRUNDPREIS + kilowattstunden * KLEINVERBRAUCHER_ARBEITSPREIS;
   }
   
   /**
    * <p>Gesamtkosten pro Jahr auf Basis des Tarifs für Großverbraucher.</p>
    * 
    * @param kilowattstunden Jahresgasverbrauch
    * @return Zu zahlender Gesamtbetrag.
    */
   static public double getKostenGrossverbrauch(int kilowattstunden) {
      return GROSSVERBRAUCHER_GRUNDPREIS + kilowattstunden * GROSSVERBRAUCHER_ARBEITSPREIS;
   }
   
   /**
    * <p>Gesamtkosten pro Jahr  auf Basis des günstigsten Tarifs.</p>
    * 
    * <p>In Abhängigkeit vom Verbrauch wird automatisch der günstigste Tarif ausgewählt.</p>
    * 
    * @param kilowattstunden Jahresgasverbrauch
    * @return Zu zahlender Gesamtbetrag.
    */
   static public double getKostenGuenstigsterTarif(int kilowattstunden) {
      return Math.min(getKostenKleinverbrauch(kilowattstunden), getKostenGrossverbrauch(kilowattstunden));
   }
}
