package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Rabatt- und Versandkostenberechnung.</p>
 * 
 * <p>Bei Bestellungen müssen folgende Punkte berücksichtigt werden:</p>
 * 
 * <ul>
 *   <li>Oberhalb eines Mindestbestellwerts erhalten Kunden
 *      einen Rabatt.</li>
 *      
 *   <li>Oberhalb eines (ggf. abweichenden) Mindestbestellwertes
 *       entfallen die Versandkosten.</li>
 *       
 *   <li>Die Rabattgrenze liegt über der Grenze für freien Versand.</li>
 * </ul>
 *
 */
public class InternetShop {
   
   /**
    * <p>Ab diesem Betrag wird ein prozentualer Rabatt in
    * Höhe von {@link #PROZENTUALER_RABATT} gewährt.</p>
    */
   static public final double RABATTGRENZE = 300.00;
   /**
    * <p>Ab diesem unrabattiertem Bestellwert erfolgt ein
    * kostenfreier Versand.</p>
    */
   static public final double GRENZE_VERSANDFREI = 100.00;
   /**
    * <p>Dieser prozentuale Rabatt wird auf Bestellungen gewährt, deren
    * Summe den Wert von {@link #RABATTGRENZE} erreicht.</p>
    */
   static public final double PROZENTUALER_RABATT = 5.0;
   /**
    * <p>Versandkosten fallen an für Bestellwerte unterhalb von
    * {@link #GRENZE_VERSANDFREI}.</p>
    */
   static public final double VERSANDKOSTEN = 3.50;
   
   /**
    * <p>Berechnung der Gesamtkosten einer Internetbestellung.</p>
    * 
    * <p>Die Gesamtkosten einer Bestellung werden in folgender Reihenfolge berechnet:</p>
    * 
    * <ol>
    * 
    *   <li>Falls der Bestellwert mindestens {@link #RABATTGRENZE} beträgt, wird
    *     anteilmäßig {@link #PROZENTUALER_RABATT} vom Bestellwert abgezogen.</li>
    *     
    *   <li>Falls der <b>originale unrabattierte</b> Bestellwert unter
    *     {@link #GRENZE_VERSANDFREI} liegt, wird der in {@link #VERSANDKOSTEN}
    *     hinterlegte Wert addiert.</li>
    * 
    * </ol>
    * 
    * <p>Beispiel: Der Kunde bestellt für 400,00 €. Da dies über der Rabattgrenze liegt,
    *   werden zunächst 5% entsprechend 20,00€ abgezogen. Da der unrabattierte
    *   Betrag über der Grenze für kostenfreien Versand liegt, ist 380,00€ zugleich
    *   auch der Endbetrag.</p>
    * 
    * @param bestellwert Die unrabattierte Summe aller bestellten Artikel.
    * @return Der Gesamtpreis gemäß Erläuterung.
    */
   static public double getGesamtpreis(double bestellwert) {
      
      if (bestellwert < GRENZE_VERSANDFREI) {
         return bestellwert + VERSANDKOSTEN;
      } else {
         if (bestellwert < RABATTGRENZE) {
            return bestellwert;
         } else {
            return (100 - PROZENTUALER_RABATT) * bestellwert / 100;
         }
      }
   }
}