package de.hdm_stuttgart.mi.sd1.test.aufgabe2;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;

import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.aufgabe2.Ersatzteil;
import de.hdm_stuttgart.mi.sd1.test.helper.ReflectClass;
import de.hdm_stuttgart.mi.sd1.test.helper.ReflectionHelper;

/**
 * Teste Ersatzteile.
 *
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_Ersatzteil extends ExaminationTestDefaults {
	
	final ReflectionHelper rHelper = new ReflectionHelper();

   /**
    * Korrekte toString() Redefinition.
    */
   @Test
   @Marking(points = 2)
   public void test_010_toString() {
      Assert.assertEquals("Holzbalken, Teilenummer:123", new Ersatzteil(123, "Holzbalken").toString());
   }
   /**
    * Korrekte toString() Redefinition.
    */
   @Test
   @Marking(points = 1)
   public void test_015_toString() {
      Assert.assertEquals("Bezeichnung unbekannt, Teilenummer:123", new Ersatzteil(123, null).toString());
   }
	/**
	 * Objektgleichheit positiv, gleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_020_equal() {
      Assert.assertEquals("Keine (geeignete) Ersatzteil.equals() Methode vorhanden!  ",
            null,
            rHelper.ersatzteil_equals.getRecursiveErrorMsg());
		Assert.assertEquals(true, new Ersatzteil(23, "Mutter").equals(
				new Ersatzteil(23, "Mutter")));
	}
	/**
	 * Objektgleichheit positiv, ungleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_030_equalBezeichnungAnders() {
      Assert.assertEquals("Keine (geeignete) Ersatzteil.equals() Methode vorhanden!  ",
            null,
            rHelper.ersatzteil_equals.getRecursiveErrorMsg());
		Assert.assertEquals(true, new Ersatzteil(23, "Mutter").equals(
				new Ersatzteil(23, "Schraube")));
	}
	/**
	 * Objektgleichheit negativ, gleiche Bezeichnung
	 */
	@Test
	@Marking(points = 1)
	public void test_040_verschieden() {
	   
      Assert.assertNull(rHelper.ersatzteil_equals.getRecursiveErrorMsg());
      Assert.assertNull(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class));

		Assert.assertEquals(false, new Ersatzteil(23, "Mutter").equals(
				new Ersatzteil(24, "Mutter")));
	}
	/**
	 * Objektgleichheit negativ, ungleiche Bezeichnung
	 */
	@Test
	@Marking(points = 2)
	public void test_050_verschiedenBezeichnung() {	   
      Assert.assertNull(rHelper.ersatzteil_equals.getRecursiveErrorMsg());
      Assert.assertNull(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class));

      Assert.assertEquals(false, new Ersatzteil(23, "Mutter").equals(
				new Ersatzteil(24, "Schraube")));
	}
	/**
	 * Objektgleichheit negativ, Vergleich mit null
	 */
	@Test
	@Marking(points = 2)
	public void test_060_verschiedenNull() {
      Assert.assertNull(rHelper.ersatzteil_equals.getRecursiveErrorMsg());
      Assert.assertNull(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class));

      Assert.assertEquals(false, new Ersatzteil(23, "Mutter").equals(null));
	}
	/**
	 * Objektgleichheit negativ, Vergleich mit String
	 */
	@Test
	@Marking(points = 1)
	public void test_070_verschiedenString() {
      Assert.assertNull(rHelper.ersatzteil_equals.getRecursiveErrorMsg());
      Assert.assertNull(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class));

      Assert.assertEquals(false, new Ersatzteil(23, "Mutter").equals("irgendwas"));
	}

   /**
    * Alle Ersatzteil Attribute sollen private sein.
    */
   @Test
   @Marking(points = 1)
   public void test_075_attrib_ersatzteil_private() {
      
      Assert.assertTrue("Die Klassse 'Ersatzteil' hat kein Attribut", 0 < Ersatzteil.class.getDeclaredFields().length);
      
      final String nonPrivateAttribs = ReflectClass.getNonPrivateAttributes(Ersatzteil.class);
      Assert.assertEquals("Folgende Attribute in der Klasse 'Ersatzteil' sind nicht 'private':  " + nonPrivateAttribs + ".   ",
            null,
            nonPrivateAttribs);
   }	
	
	/**
	 * Definition class Reifen {...}
	 */
	@Test
	@Marking(points = 1)
	public void test_080_classReifen() {
		Assert.assertEquals(null, rHelper.reifenKlasse.msg);
	}
	
	/**     modifier & method.getModifiers();

	 * Konstruktor Definition public Reifen (int, int){...}
	 */
	@Test
	@Marking(points = 1)
	public void test_200_reifenKonstruktor() {
		Assert.assertEquals(null, rHelper.reifen_constructor_int_int.getRecursiveMsg());
	}

	/**
	 * Methode Ersatzteil.equals() in Ersatzteil darf nicht redefinierbar sein
	 */
	@Test
	@Marking(points = 1)
	public void test_077_equalsNichtRedefinierbar() {
		Assert.assertNull(rHelper.ersatzteil_equals.getRecursiveErrorMsg());
      Assert.assertNull(rHelper.ersatzteil_equals.assertIsDefinedIn(Ersatzteil.class));
      Assert.assertNull(rHelper.ersatzteil_equals.assertModifierIsPresent(Modifier.FINAL, "Methode Ersatzteil.equals() darf nicht redefinierbar sein"));

	}

	/**
	 * Methode Reifen.toString() mit korrekter Signatur definiert
	 */
	@Test
	@Marking(points = 1)
	public void test_220_Reifen_toString() {		
		Assert.assertEquals(null, rHelper.reifen_toString.getRecursiveErrorMsg());

      Assert.assertNull(rHelper.reifen_toString.assertIsDefinedIn("de.hdm_stuttgart.mi.sd1.aufgabe2.Reifen"));
      Assert.assertNull(rHelper.reifen_toString.assertModifierIsPresent(Modifier.PUBLIC));
      
	}

	/**
	 * Erzeugung Instanz Reifen(final int number) und pruefung toString()
	 */
	@Test
	@Marking(points = 2)
	public void test_400_konstruktorReifen() {
      Assert.assertEquals(null, rHelper.reifenKlasse.msg);
		Assert.assertEquals(null, rHelper.reifen_constructor_int_int.getRecursiveMsg());
		
		Object reifen = null;
		try {
			reifen = rHelper.reifen_constructor_int_int.constructor.newInstance(new Object[] {1132, 160});
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			Assert.fail("Konnte 'new Reifen(1132, 160)' nicht aufrufen.");
			return;
		}
		Assert.assertTrue("Objekt ist kein Ersatzteil", reifen instanceof Ersatzteil);
		try {
			Assert.assertEquals("Reifen, Teilenummer:1132, Maximalgeschwindigkeit:160 Kmh", rHelper.reifen_toString.method.invoke(reifen, new Object[] {}));
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			Assert.fail("Konnte 'toString()' für 'new Reifen(1132, 160)' nicht aufrufen.");
			return;
		}
	}
}