package de.hdm_stuttgart.mi.sda2.catalog;

import java.io.File;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import de.hdm_stuttgart.mi.sda2.catalog.domain.Product;
import de.hdm_stuttgart.mi.sda2.catalog.domain.ProductProvider;
import de.hdm_stuttgart.mi.sda2.catalog.jaxb.Article;
import de.hdm_stuttgart.mi.sda2.catalog.jaxb.Group;
import de.hdm_stuttgart.mi.sda2.catalog.jaxb.ProductGroups;

/**
 * Loading XML and RDBMS data
 * 
 */
public class LoadXmlRdbms {

   /**
    * @param args Unused
    */
   @SuppressWarnings("javadoc")
   public static void main(String[] args) throws JAXBException {

      final EntityManagerFactory factory = Persistence
            .createEntityManagerFactory("persistenceUnitNoDrop");
      final EntityManager manager = factory.createEntityManager();


      
      final JAXBContext cdContext = JAXBContext.newInstance("de.hdm_stuttgart.mi.sda2.catalog.jaxb");
      final Unmarshaller u = cdContext.createUnmarshaller();
      final ProductGroups productGroups= (ProductGroups) u.unmarshal(new File("XmlData/samplegroups.xml"));

      System.out.println(productGroups.getGroup().size() + " product group(s):");
      for (final Group g : productGroups.getGroup()) {
         System.out.println("Group title:" + g.getTitle() + " containing " + g.getArticle().size() + " article(s)");
         
         for (Article a : g.getArticle()) {
            System.out.println( "Article no. " + a.getProductNumber());
            addRdbmsData(manager, a.getProductNumber());
         }
         System.out.println("----------------");
      }
   }
   
   private static void addRdbmsData(final EntityManager em, final int productNumber) {
      final EntityTransaction tx = em.getTransaction();
      tx.begin();
      {
         
         final TypedQuery<Product> query = em.createQuery(
               "SELECT p FROM Product p WHERE p.productNumber = :productNumber", Product.class);
         query.setParameter("productNumber", productNumber);
         final List<Product> retrievedProducts = query.getResultList();
         if (0 == retrievedProducts.size()) {
            System.out.println("No product data available");
         } else {
            final Product p = retrievedProducts.get(0);
            System.out.print(p.toString());
            ProductProvider pp = p.getPrimaryProvider();
            if (null == pp) {
               System.out.println();
            } else {
               System.out.println(", primary provider:" + pp.getName());
            }
         }
           
      }
      tx.commit();
   }
}
