package de.hdm_stuttgart.mi.sd1.store;

/**
 * A container holding a fixed
 *  number of integer values.
 *
 */
public class IntegerStore {
  
  final static int defaultCapacity = 4;
  int[] values ;         // Array containing our values
  int numValues = 0;     // Number of values present in the container so far.
  
  /**
   * Create a new store being able to
   * hold integer values.
   * 
   */
  public IntegerStore() {
    values = new int[defaultCapacity];
  }
  /**
   * Create a new store being able to
   * hold integer values.
   * 
   * @param capacity The store's initial capacity. If more values
   * are being added via {@link #addValue(int)} the capacity
   * will be increased dynamically.
   * 
   */
  public IntegerStore(int capacity) {
    values = new int[capacity];
  }

  /**
   * @return The number of elements the store may
   * hold, see {@link #IntegerStore(int)}.
   */
  public int getCapacity() {
    return values.length;
  }
  
  /**
   * @return The number of values being contained.
   */
  public int getNumValues() { 
    return numValues;
  }
  
  /**
   * Insert a new value into our container
   * 
   * @param value The value to be inserted. This will increment
   * {@link #getNumValues()} by one.
   * 
   */
  public void addValue(int value) {
    if (values.length <= numValues) { // Sufficient capacity available to add a new value?
      final int[] currentArray = values;
      values = new int[2 * currentArray.length]; // Double the current array's size.
      for (int i = 0; i < currentArray.length; i++) {
        values[i] = currentArray[i];
      }
    } 
    values[numValues++] = value; 
  }

  /**
   * Access the value at a given index
   * 
   * @param index The desired value's index
   * @return The desired value. Precondition: index &lt; {@link #getNumValues()}}
   * 
   */
  public int getValue(int index) {
    return values[index];
  }
  
  /**
   * Empty the current set of values and set the store
   * to its initial state.
   */
  public void clear() {
    numValues = 0;
  }
  
  /**
   * @return The given sample's arithmetic average
   */
  public double getAverage() {
    double sum = 0;
    for (int i = 0; i < numValues; i++) {
      sum += values[i];
    }
    return sum / numValues;
  }
}