package de.hdm_stuttgart.mi.sd1;

import java.util.Arrays;

public class Mask {

  /**
   * Number of character positions being visible when masking a string. See {@link #mask(String)}
   * for details.
   */
  static public final int VISIBLE_POSITIONS = 4;

  /**
   * <p>Mask all but {@link #VISIBLE_POSITIONS} positions of a given string by '#' characters. Examples:</p>
   *
   * <ul>
   *   <li> "You"  --> "You" </li>
   *   <li> "Secret"  --> "##cret" </li>
   *   <li> "Ultimate test"  --> "#########test" </li>
   * </ul>
   *
   * @param s The input string to be masked.
   * @return The masked string or null in case of null input string
   */
  public static String mask(final String s) {
    if (null == s || s.length() <= VISIBLE_POSITIONS) {
      return s;
    } else {
      final int maskLength = s.length() - VISIBLE_POSITIONS;

      final StringBuffer result = new StringBuffer();
      for (int i = 0; i < maskLength; i++) {
        result.append('#');
      }
      return result.append(s.substring(maskLength)).toString();
    }
  }

  /**
   * Alternate implementation using yet not covered array topic.
   *
   * @param s The input string to be masked.
   * @return The masked string or null in case of null input string
   */
  public static String mask2(final String s) {
    if (null == s || s.length() <= VISIBLE_POSITIONS) {
      return s;
    } else {
      final int maskLength = s.length() - VISIBLE_POSITIONS;
      final char[] mask = new char[maskLength];
      Arrays.fill(mask, '#');
      return new String(mask).concat(s.substring(maskLength));
    }
  }
}
