package de.hdm_stuttgart.mi.sd1.htmlformat;

/**
 * A single address record holding a representation as being
 * described in {@link AddressDataHandler#AddressDataHandler(String)}
 *
 */
public class Address {

  final String firstName, lastName, companyName,
  address, city, county, postal,
  phone1, phone2, email, web;


  /**
   * Initializing from an address string obeying the description given
   * in  {@link AddressDataHandler#AddressDataHandler(String)}.
   * 
   * @param csvRecord An input address record like
   *   <code>"Aleshia","Tomkies","Alan Co.","14 Taylor St",...</code>
   * @param lineNumber Indication the current file position in case of parsing errors.
   * @throws AddressParseError Data source is corrupt.
   */
  public Address(final String csvRecord, final int lineNumber) 
      throws AddressParseError {
    if (!csvRecord.startsWith("\"")) {
      throw new  AddressParseError(
          "Record does not start with \":\n" + csvRecord, lineNumber);
    }
    if (!csvRecord.endsWith("\"")) {
      throw new  AddressParseError(
          "Record does not end with \":\n" + csvRecord, lineNumber);
    }
    // Split input record at "," positions.
    final String[] csvComponents = csvRecord.substring(1, csvRecord.length() - 1).split("\",\"");
    if (11 == csvComponents.length) {// Does the current record have 11 address components?
      
      firstName = csvComponents[0];
      lastName = csvComponents[1];
      companyName = csvComponents[2];
      address = csvComponents[3];
      city = csvComponents[4];
      county = csvComponents[5];
      postal = csvComponents[6];
      phone1 = csvComponents[7];
      phone2 = csvComponents[8];
      email =   csvComponents[9];
      web = csvComponents[10];
    } else { // more or less than 11 components
      throw new  AddressParseError(
          "Record does not contain 11 address components:\n" + csvRecord,
          lineNumber);
    }
  }
}
