package htmlformat;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.sd1.htmlformat.AddressDataHandler;
import de.hdm_stuttgart.mi.sd1.htmlformat.AddressParseError;

/**
 * Testing address file parsing exceptions
 *
 */
public class TestExceptions {

  /**
   * Test file "Testdata/missingAttribute.txt" contains a record with missing last component.
   */
  @Test
  public void testMissingAttribute() {
    final String data = "Testdata/missingAttribute.txt";
    AddressParseError parseException = null;
    try {
      new AddressDataHandler(data);
    } catch (FileNotFoundException e) {
      Assert.fail("File " + data  + "should be readable");
    } catch (IOException e) {
      Assert.fail("File " + data  + "should be readable");
    } catch(AddressParseError e) {
      parseException = e;
    }
    Assert.assertNotNull("Error was expected alt line ", parseException);
    Assert.assertEquals("Error was expected at line 3", 3, parseException.getErrorOffset());
    Assert.assertTrue(parseException.getMessage().startsWith("Record does not contain"));
  }
  
  /**
   * Test file "Testdata/missingStartQuotes.txt" contains a record with missing quote at beginning.
   */
  @Test
  public void testMissingStartQuote() {
    final String data = "Testdata/missingStartQuote.txt";
    AddressParseError parseException = null;
    try {
      new AddressDataHandler(data);
    } catch (FileNotFoundException e) {
      Assert.fail("File " + data  + "should be readable");
    } catch (IOException e) {
      Assert.fail("File " + data  + "should be readable");
    } catch(AddressParseError e) {
      parseException = e;
    }
    Assert.assertNotNull("Error was expected", parseException);
    Assert.assertEquals("Error was expected at line 4", 4, parseException.getErrorOffset());
    Assert.assertTrue(parseException.getMessage().startsWith("Record does not start with \":"));
  }
  // Two more test cases but I'm too lazy tonight.
}
