package de.hdm_stuttgart.mi.sd1.grep;

/**
 * Filter a given set of input lines- This class and its descendants
 * could be avoided in favor of Java 8 lambda expressions.
 */
public abstract class LineFilter {

	/**
	 * To be matched as part of an input line.
	 */
	protected final String filterString;

	/**
	 * Defining a string to be contained within lines of input.
	 * 
	 * @param filterString This string must be contained
	 *  in a given line of input to be accepted by {@link #accept(String)}. Matches may
	 *  possibly be case insensitive depending an the concrete class being used.
	 *  
	 */
	public LineFilter(final String filterString) {
		this.filterString = filterString;
	}

	/**
	 * Will the given line of input become part of the result set?
	 * 
	 * @param line The input text to be matched
	 * 
	 * @return true if input contains the {@link #filterString}), false otherwise.
	 */
	abstract boolean accept(final String line);
	/**
	 * Print each matching input line to standard output.
	 * 
	 * @param line The input text to be matched
	 * @param prefix To be prepended before each line of output e.g.
	 *              blahblah.txt: <matched line>
	 * 
	 * @return true if input contains the {@link #filterString}), false otherwise.
	 */
	final void acceptAndPrint(String line, String prefix) {
		if (accept(line)) {
			System.out.println(prefix + line);
		}
	}

}
