package de.hdm_stuttgart.mi.sd1.textstat;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Count words and their respective frequencies of appearance.
 *
 */
public class TextFileHandler {

  /**
   * The input file name to read from.
   */
  public final String filename;
  
  /**
   * The list of words found so far among with their frequencies.
   */
  final Map<String, Integer> wordFrequencies = new Hashtable<String, Integer>();
  
  /**
   * @param filename The filename input to read input, see {@link #filename}
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   */
  public TextFileHandler(final String filename)
      throws FileNotFoundException, IOException {
    
    this.filename = filename;
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    for (String line = reader.readLine(); line != null; line = reader.readLine()) {
      // Ignore white space and some common text markers.
      final String[] words = line.split("[ \t\"!?.,'´`]+");
      for (final String word: words) {
        
        // Word might be new
        final Integer existingFrequency = wordFrequencies.get(word);
        if (null == existingFrequency) { // Word is not yet in list.
          wordFrequencies.put(word, new Integer(1));
        } else { // Word is already in our list. Replace by frequency+1 value.
          wordFrequencies.put(word, existingFrequency + 1);
        }
      }
    }
    reader.close();
  }
  
  /**
   * Print an unsorted list of all words
   */
  public void printFrequencies() {
    for (final Entry<String, Integer> entry : wordFrequencies.entrySet()) {
      System.out.format("%3d: %s\n",
                         entry.getValue(),
                              entry.getKey());
    }
  }
}
