package de.hdm_stuttgart.mi.sd1.textstat;

import java.util.Comparator;
import java.util.Map.Entry;

/**
 * Comparing word frequencies first by frequency (descending) and second
 * alphabetically ignoring case (ascending).
 * 
 * Consider three entry examples ("software"|35),
 * ("function"|35) and ("error"|10). These entries
 * reflect the words "software", "function" and "error" appearing in a text file with
 * frequencies 35, 35 and 10 respectively.
 * 
 * The intended sorting order should be:
 * <ul>
 *   <li>("function"|35)</li>
 *   <li>("software"|35)</li>
 *   <li>("error"|10)</li>
 * </ul>
 * 
 *
 */
public class WordFrequencyCompare implements Comparator<Entry<String, Integer>> {

	@Override
	public int compare(Entry<String, Integer> e1, Entry<String, Integer> e2) {
		
		if (e1.getValue().equals(e2.getValue())) { // Word frequencies are equal
			// Just compare the strings in question alphabetically.
			return e1.getKey().toLowerCase().compareTo(e2.getKey().toLowerCase());
		} else {
			return e2.getValue() - e1.getValue(); // Sort by frequency value
		}
	}
}
