package de.hdm_stuttgart.mi.sd1.textstat;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Collect words from a text.
 *
 */
public class TextFileHandler {

  /**
   * The set of words found so far.
   */
  final SortedSet<String> words = new TreeSet<String>();
  
  /**
   * @param filename The filename input to read input.
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   */
  public TextFileHandler(final String filename)
      throws FileNotFoundException, IOException {
    
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    for (String inputLine = reader.readLine(); inputLine != null; inputLine = reader.readLine()) {
      // Ignore white space and some common text markers.
      final String[] inputWords = inputLine.split("[ \t\"!?.,'´`:;]+");
      for (final String word: inputWords) {
        if (0 < word.trim().length()) { // Exclude empty text strings at beginning or end of line
          words.add(word);
        }
      }
    }
    reader.close(); // Free OS resources no longer being needed. 
  }
  
  
  
  /**
   * @return The set of words being found.
   */
  public Set<String> getWords() {
    return words;
  }



  /**
   * 
   * @return A comma separated list of words being found.
   */
  public String getCsWordList() {
    StringBuffer buffer = new StringBuffer();
    
    String separator = "";
    for (final String word: words) {
      buffer.append(separator + word);
      separator = ", ";
    }
    return buffer.toString();
  }
}