package de.hdm_stuttgart.mi.sd1.plot;

/**
 * 2-dimensional sine function plot
 *
 */
public class Plotter {

   int  // Effective size of plotting region
   numTicsX  // Number of plotting positions in x-direction
   ,numTicsY; // Number of plotting positions in y-direction

   double 
   xMin			// x-values ranging from xMin to xMax
   ,xMax

   ,yMin			// y-values ranging from yMin to yMax
   ,yMax 

   ,xStepWidth   // per step width in x-direction
   ,yStepWidth;  // per step width in y-direction

   char [][] plotRegion;
   /**
    * The plot regions character size like e.g.
    *  "30 lines by 120 characters each"
    * 
    * @param numTicsX Number of characters per line
    * @param numTicsY Number of lines
    */
   public void setNumTics(int numTicsX, int numTicsY) {
      this.numTicsX = numTicsX;
      this.numTicsY = numTicsY;
   }

   /**
    * Argument range to be plotted
    * i.e sin(x),  x in the range of [0, 2 * PI[
    * 
    * @param xMin
    * @param xMax
    */
   public void setXrange(final double xMin, final double xMax) {
      this.xMin = xMin;
      this.xMax = xMax;
   }

   /**
    * Value range to be plotted
    * i.e sin(x) having values in the range of [-1, 1]
    * 
    * @param yMin lowest function value
    * @param yMax highest function value
    */
   public void setYrange(final double yMin, final double yMax) {
      this.yMin = yMin;
      this.yMax = yMax;
   }

   void init() { // Initialize plotting region by spaces
      xStepWidth = (xMax - xMin) / (numTicsX - 1);
      yStepWidth = (yMax - yMin) / (numTicsY - 1);
      plotRegion = new char[numTicsX][numTicsY];

      for (int xTics = 0; xTics < numTicsX; xTics++) {
         for (int yTics = 0; yTics < numTicsY; yTics++) {
            plotRegion[xTics][yTics] = ' ';
         }
      }
   }

   /**
    * Plot a function. Ranges and plot sizes must be set accordingly
    * by previously calling {@link #setNumTics(int, int)}, {@link #setXrange(double, double)}
    * and {@link #setYrange(double, double)}
    * 
    * @param f The function to be plotted
    */
   public void plot(final DoubleOfDoubleFunction f) {
      init();

      for (int xTics = 0; xTics < numTicsX; xTics++) {
         double x = xMin + xTics * xStepWidth;
         double functionValue = f.compute(x);

         if (yMin <= functionValue && functionValue <= yMax ) {
            int ySinTics = (int) Math.round((functionValue - yMin) / (yMax - yMin) * (numTicsY - 1));
            plotRegion[xTics][ySinTics] = '*';
         }
      }
      for (int yTic = numTicsY - 1; 0 <= yTic; yTic--) {
         for (int xTic = 0; xTic < numTicsX; xTic++) {
            System.out.print(plotRegion[xTic][yTic]);
         }
         System.out.print('\n'); // Newline
      }
   }
}
