package de.hdm_stuttgart.mi;

import javax.servlet.annotation.WebServlet;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.annotations.Widgetset;
import com.vaadin.event.ShortcutAction.KeyCode;
import com.vaadin.server.ClientConnector.DetachListener;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.ui.Button;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.themes.ValoTheme;

import de.hdm_stuttgart.mi.persistence.PersistView;
import de.hdm_stuttgart.mi.persistence.PersistenceHandler;

/**
 *
 */
@Theme("mytheme")
@Widgetset("de.hdm_stuttgart.mi.MyAppWidgetset")
public class LoginUI extends UI implements DetachListener, PersistView {

   private static final long serialVersionUID = 9013400739498375365L;
   
   final PersistenceHandler persistenceHandler = 
         new PersistenceHandler(this);

   private final TextField
   nameField = new TextField(), 
   emailField = new TextField();

   private Button
   toggleConnectButton = new Button("Connect"),
   insertButton = new Button("Insert");

   final GridLayout databaseFieldLayout = new GridLayout(2, 3);
   final VerticalLayout panelLayout = new VerticalLayout(toggleConnectButton, databaseFieldLayout);

   /* (non-Javadoc)
    * @see de.hdm_stuttgart.mi.P#setStatusConnected()
    */
   @Override
   public void setStatusConnected() {
      toggleConnectButton.setCaption("Disconnect");
      toggleConnectButton.setDescription("Disconnect from database server");
      nameField.setEnabled(true);
      emailField.setEnabled(true);
   }

   /* (non-Javadoc)
    * @see de.hdm_stuttgart.mi.P#setStatusDisconnected()
    */
   @Override
   public void setStatusDisconnected() {
      toggleConnectButton.setCaption("Connect");
      toggleConnectButton.setDescription("Connect to database server");
      databaseFieldLayout.iterator().forEachRemaining(it -> it.setEnabled(false));
   }
   /* (non-Javadoc)
    * @see de.hdm_stuttgart.mi.P#setDatasetInserted()
    */
   @Override
   public void setDatasetInserted() {
      nameField.clear();
      emailField.clear();
   }
   
   @Override
   public void guiInfo(final String msg) {
      Notification.show("Info",
            msg,
            Notification.Type.HUMANIZED_MESSAGE);
   }

   @Override
   public void guiError(final String msg) {
      Notification.show("Error",
            msg,
            Notification.Type.ERROR_MESSAGE);
   }

   /**
    * 
    */
   public LoginUI () {}

   @Override
   public void detach(DetachEvent event) {
      if (persistenceHandler.isConnected()) {
         persistenceHandler.toggleConnectionState();
      }
   }

   void conditionallyActivateInsertButton() {
      final boolean isValid = 
            0 < nameField.getValue().trim().length() &&
            0 < emailField.getValue().trim().length();

      insertButton.setEnabled(isValid);
   }

   @Override
   protected void init(final VaadinRequest vaadinRequest) {

      setContent(panelLayout);
      databaseFieldLayout.setMargin(true);

      // Assembling GUI components
      databaseFieldLayout.addComponent(new Label("Name:"), 0, 0);
      databaseFieldLayout.addComponent(nameField, 1, 0);

      databaseFieldLayout.addComponent(new Label("Email:"), 0, 1);
      databaseFieldLayout.addComponent(emailField, 1, 1);

      databaseFieldLayout.addComponent(insertButton, 1, 2);

      // Inserting requires valid user input.
      insertButton.setEnabled(false);
      insertButton.setClickShortcut(KeyCode.ENTER);
      insertButton.addStyleName(ValoTheme.BUTTON_PRIMARY);

      // Check for global user input validity 
      nameField.addValueChangeListener(event -> conditionallyActivateInsertButton());
      emailField.addValueChangeListener(event -> conditionallyActivateInsertButton());
            
      toggleConnectButton.addClickListener(
            event -> persistenceHandler.toggleConnectionState() );

      insertButton.addClickListener(event ->
         persistenceHandler.insert(nameField.getValue(), emailField.getValue())
      );
   }

   /**
    *
    */
   @WebServlet(urlPatterns = "/*", name = "LoginUIServlet", asyncSupported = true)
   @VaadinServletConfiguration(ui = LoginUI.class, productionMode = false)
   public static class LoginUIServlet extends VaadinServlet {
      private static final long serialVersionUID = -8657597788907698974L;
   }
}
