package de.hdm_stuttgart.mi;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import javax.servlet.annotation.WebServlet;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.annotations.Widgetset;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.server.ClientConnector.DetachListener;
import com.vaadin.ui.Button;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Notification;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;

@Theme("mytheme")
@Widgetset("de.hdm_stuttgart.mi.MyAppWidgetset")
public class LoginUI extends UI implements DetachListener {

   private static final long serialVersionUID = 9013400739498375365L;
   
   private static Logger log = LogManager.getLogger(LoginUI.class);

   Statement stmt;

	public LoginUI () {
	   final String jdbcUrl = "jdbc:mysql://localhost:3306/hdm?useSSL=false";
	   try {
		final Connection conn = DriverManager.getConnection(
				jdbcUrl, "hdmuser", "XYZ");
		stmt = conn.createStatement();
	   } catch (SQLException ex) {
	      log.error("Unable to connect to database using '" + jdbcUrl + "':", ex);
	      stmt = null;
	   }
	}
	
	@Override
	public void detach(DetachEvent event) {
	   if (null != stmt) {
	      try {
	         stmt.getConnection().close();
	      } catch (final SQLException ex) {
	         log.error("Unable to disconnect from database:", ex);
	      }
	      stmt = null;
	   }
	}

	private final TextField
	   nameField = new TextField(), 
	   emailField = new TextField();

	private Button
	   insertButton = new Button("Insert");

	@Override
	protected void init(final VaadinRequest vaadinRequest) {
		final GridLayout layout = new GridLayout(2, 3);
		layout.setMargin(true);
		setContent(layout);

      if (null == stmt) {
         layout.addComponent(new Label("Unable to connect to database server, see log for details"), 0, 0);
         return;
      }

		layout.addComponent(new Label("Name:"), 0, 0);
		layout.addComponent(nameField, 1, 0);

		layout.addComponent(new Label("Email:"), 0, 1);
		layout.addComponent(emailField, 1, 1);

		layout.addComponent(insertButton, 1, 2);
		
		
		insertButton.addClickListener(event -> {
		   final String sql = "INSERT INTO Person VALUES('" + nameField.getValue() + "', '" 
               + emailField.getValue() +  "')";
         // We have to swallow this Exception because an ActionListener's signature
         // does not allow for a "throws" clause.
         try {
            final int updateCount = stmt.executeUpdate(sql);
            Notification.show("Successfully executed \n'" + sql + "'\ninserteing "
                  + updateCount + " dataset");

         } catch (SQLException ex) {
            Notification.show("Failed to insert record, see log for details", Notification.Type.ERROR_MESSAGE);
            log.error("JDBC exception on data insert:", ex);
         }		   
		});
	}

	@WebServlet(urlPatterns = "/*", name = "LoginUIServlet", asyncSupported = true)
	@VaadinServletConfiguration(ui = LoginUI.class, productionMode = false)
	public static class LoginUIServlet extends VaadinServlet {
		private static final long serialVersionUID = -8657597788907698974L;
	}
}
