package de.hdm_stuttgart.mi.sda1.xml;

import de.hdm_stuttgart.mi.sda1.Db.WriteRelationalData;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.input.SAXBuilder;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class ReadData {

  static private final SAXBuilder builder = new SAXBuilder();

  private final Element lecturers, courses;

  private static final Map<String, Object> xpathVarsNamespacePrefix = new HashMap<>();

  static {
    xpathVarsNamespacePrefix.put("ident", null);
  }

  private final XPathExpression<Element> courses2lecturer;

  public ReadData(final String inputFile) throws JDOMException, IOException {
    final Element university = builder.build(new File(inputFile)).getRootElement();
    lecturers = university.getChild("lecturers");
    courses = university.getChild("courses");

    courses2lecturer = XPathFactory.instance().compile(
      "course[@lecturer = $ident]",
      new ElementFilter(),
      xpathVarsNamespacePrefix,
      new Namespace[]{});
  }

  public void transfer(final WriteRelationalData dest) throws SQLException {
    dest.eraseData();
    for(Element lecturer: lecturers.getChildren("lecturer")){ // No stream due to SQLException, sigh!
      insertLecturer(lecturer, dest);
    }
  }

  private void insertLecturer(final Element lecturer, final WriteRelationalData dest) throws SQLException {

    final String
      ident = lecturer.getAttributeValue("ident"),
      email = lecturer.getAttributeValue("email"),
      name = lecturer.getChild("name").getTextTrim();

    final long lecturerId = dest.insertLecturer(ident, name, email);
    insertCoursesOfLecturer(lecturerId, ident, dest);
  }

  private void insertCoursesOfLecturer(
    final long lecturerId, final String lecturerIdent, final WriteRelationalData dest) throws SQLException {

    courses2lecturer.setVariable("ident", lecturerIdent);

    for (final Element c: courses2lecturer.evaluate(courses)) {
      dest.insertCourse(
        Integer.parseInt(c.getAttributeValue("number")),
        lecturerId,
        c.getChildText("title"),
        c.getChildText("description"));
    }
  }
}