package de.hdm_stuttgart.mi.persistence.test;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.persistence.DbProps;
import de.hdm_stuttgart.mi.persistence.PersistenceHandler;

/**
 *
 */
public class DataInsertTest {
   
   final ViewMockup viewMockup = new ViewMockup();

   final PersistenceHandler ph = new PersistenceHandler(viewMockup);

   /**
    */
   @Test ()
   public void addDatasets() {
      try {
         initDatabase();
      } catch (SQLException e) {
         Assert.fail("Unable to initialize database:" + e.getLocalizedMessage());
         return;
      }
      ph.toggleConnectionState();
      Assert.assertTrue(ph.insert("Anton", "a@mail.com"));

      // Failure expected due to primary key constraint violation
      Assert.assertFalse(ph.insert("Anton", "a@mail.com"));
      Assert.assertNotNull(viewMockup.error);
      Assert.assertEquals(
            "E-mail already in database",
            viewMockup.error);
      
      try {
         cleanDatabase();
      } catch (SQLException e) {
         System.err.println("Unable to clean database");
      }
      Assert.assertTrue(ph.insert("Anton", "a@mail.com"));
      
   }

   private void initDatabase() throws SQLException {
      
      final Connection conn = DriverManager.getConnection(
            DbProps.getString("PersistenceHandler.jdbcUrl"),
            DbProps.getString("PersistenceHandler.username"),
            DbProps.getString("PersistenceHandler.password"));
      final Statement stmt = conn.createStatement();
      stmt.executeUpdate("DROP TABLE IF EXISTS Person");
      stmt.executeUpdate("CREATE TABLE Person (\n" +
                            "name char(80)\n" + 
                            ",email CHAR(20) UNIQUE\n" +
                            ");");
      conn.close();
   }
   private void cleanDatabase() throws SQLException {
      
      final Connection conn = DriverManager.getConnection(
            DbProps.getString("PersistenceHandler.jdbcUrl"),
            DbProps.getString("PersistenceHandler.username"),
            DbProps.getString("PersistenceHandler.password"));
      final Statement stmt = conn.createStatement();
      stmt.executeUpdate("DELETE FROM Person");
      conn.close();
   }
}
