package de.hdm_stuttgart.mi.javastreams;

import java.util.EnumSet;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collector;

/**
 * Turning a list of strings into a character separated list of words. Example:
 * 
 * List {"Fred", "Eve", "Petra" } and separator "+" results
 * in the string "Fred+Eve+Petra".
 *
 */
public class CsvCollector implements Collector<String, StringBuffer, String> {
   
   final String separator;
   
   /**
    * Providing the list's desired separator string.
    * @param separator String filling the gap between two adjacent strings.
    */
   public CsvCollector(final String separator) {
      this.separator = separator;
   }
   
   @Override
   public Supplier<StringBuffer> supplier() {
      return () -> new StringBuffer();
   }

   @Override
   public BiConsumer<StringBuffer, String> accumulator() {
      return (csv, s) -> {
         if (0 != csv.length()) {   // Buffer non-empty?
            csv.append(separator);  // ==> Append separator ...
         }
         csv.append(s);             // and string.
      };
   }

   @Override
   public BinaryOperator<StringBuffer> combiner() {
      return (csv1, csv2) -> {
         if (0 == csv1.length()) {        // Left list empty?
            return csv2;                  // ==> return right list.
            
         } else if (0 == csv2.length()) { // Right List empty?
            return csv1;                  // ==> return left list.
            
         } else {                         // Both lists non-empty?
            return csv1.                  // ==> Concatenate both lists.
                  append(separator).
                  append(csv2);  
         }
      };
   }

   @Override
   public Function<StringBuffer, String> finisher() {
      return csv -> csv.toString();
   }

   @Override
   public Set<java.util.stream.Collector.Characteristics> characteristics() {
      return EnumSet.of(Characteristics.UNORDERED);
   }

}
