package de.hdm_stuttgart.mi.javastreams;

/*
 * Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */ 

import java.util.Arrays;
import java.util.List;

/** Representing students among with examination marks. */
public class Student {

   /** @return Test sample of students. */
   public static List<Student> createRoster() { // Create test data records

      final Student[] roster = new Student[] {
            new Student("Fred", 2, Student.Sex.MALE, "fred@example.com")
            ,new Student("Jane", 1, Student.Sex.FEMALE, "jane@kiv.de")
            ,new Student("George", 4, Student.Sex.MALE, "george@math.edu")
            ,new Student("Bob", 2, Student.Sex.MALE, "bob@uk.edu")
            ,new Student("Kim", 2, Student.Sex.FEMALE, "wilde@serious.de")
      };

      return Arrays.asList(roster);
   }

   /** Male or female */
   public enum Sex {
      /** */
      MALE("male"), 
      /** */
      FEMALE("female");

      final String extern;

      private Sex(final String extern) {this.extern = extern;}

      @Override
      public String toString(){return extern;}
   }

   String name; 
   int mark;
   Sex gender;
   String emailAddress;

   Student(String name, int mark ,
         Sex gender, String email) {
      this.name = name;
      this.mark = mark;
      this.gender = gender;
      this.emailAddress = email;
   }  

   /** * @return An examination's result */
   public int getMark() {
      return mark;
   }

   /** @return A student's sex. */
   public Sex getSex() {
      return gender;
   }

   /** @return A student's name. */
   public String getName() {
      return name;
   }

   /** @return A student's e-mail. */
   public String getEmailAddress() {
      return emailAddress;
   }

   /**
    * Compare two students by their marks
    * @param a first student
    * @param b second student
    * @return positive if student b's mark is better than student a's,
    *         zero if marks are equal, negative if ...
    */
   public static int compareByMark(final Student a, final Student b) {
      return a.mark - b.mark;
   }

   @Override
   public String toString() {
      return getName() + "(" + gender + ", " + getEmailAddress() + ", mark=" + getMark() + ")";
   }
}