package de.hdm_stuttgart.mi.sudoku.model;

import java.util.HashSet;
import java.util.Set;


public class SudokuBoard {

	static public final String 
	ERRMSG_3CELL_DUPLICATE = "3x3 cell has duplicate",
    ERRMSG_ROW_DUPLICATE = "duplicate in row",
	ERRMSG_COL_DUPLICATE = "duplicate in COLUMN"
	;

	static public final int size = 9;

	static private final String data =
			/**/    "000" + "260" + "701" +
					"680" + "070" + "090" +
					"190" + "004" + "500" +
					//----+-------+-------+
					"820" + "100" + "040" +
					"004" + "602" + "900" +
					"050" + "003" + "028" +
					//----+-------+-------+
					"009" + "300" + "074" +
					"040" + "050" + "036" +
					"703" + "018" + "000";

	 final IntegerBuffer [][] parseData() {

		final IntegerBuffer[][] cells = new IntegerBuffer[size][];
		for (int y = 0; y < size; y++) {
			final IntegerBuffer[] row = new IntegerBuffer[size];
			cells[y] = row;
			for (int x = 0; x < size; x++) {
				final Integer intVal = new Integer(data.charAt(x + y * size) - '0');

				if (0 == intVal) {
					row[x] = new IntegerBuffer(false, null);// "free" sudoku position, set null  ==> will later be editable
				} else {
					row[x] = new IntegerBuffer(true, intVal); // preset sudoku position ==> is not editable
				}
			}
		}
		return cells;
	}

	public String checkValue(final int value, final int yCoordinate, final int xCoordinate) {


		// Check values in 3 x 3 subwindow
		final Set<Integer> checkDuplicates = new HashSet<Integer>();
		final int 
		xSubwinBase = (xCoordinate / 3) * 3,
		ySubwinBase = (yCoordinate / 3) * 3;

		for (int y = ySubwinBase; y < ySubwinBase + 3; y++) {
			for (int x = xSubwinBase; x < xSubwinBase + 3; x++) {
				if (x != xCoordinate || y != yCoordinate) {
					cells[y][x].value.ifPresent(checkDuplicates::add);
				}
			}
		}
		if (checkDuplicates.contains(value)) {
			return ERRMSG_3CELL_DUPLICATE;
		}

		// check for y - row 
		checkDuplicates.clear();
		for (int x = 0; x < size; x++) {
			if (x != xCoordinate) {
				cells[yCoordinate][x].value.ifPresent(checkDuplicates::add);
			}
		}
		if (checkDuplicates.contains(value)) {
			return ERRMSG_ROW_DUPLICATE;
		}

		// check for x - column
		checkDuplicates.clear();
		for (int y = 0; y < size; y++) {
			if (y != yCoordinate) {
				cells[y][xCoordinate].value.ifPresent(checkDuplicates::add);
			}
		}
		if (checkDuplicates.contains(value)) {
			return ERRMSG_COL_DUPLICATE;
		}
		return null;
	}

	public final IntegerBuffer[][] cells =  parseData();

}


























