package de.hdm_stuttgart.mi.sda1.saxerrorhandler.handler;

import java.io.PrintStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * A SAX error handler providing line and column number information with respect
 * to files being parsed.
 * 
 * Output will be written to a stream
 *
 */
public class SaxErrorHandler implements ErrorHandler {

	private static final Logger log = LogManager.getLogger(SaxErrorHandler.class);

	final PrintStream problemOutputStream;

	/**
	 * Parsing problems will only published to internal logger.
	 */
	public SaxErrorHandler() {
		problemOutputStream = null;
	}

	/**
	 * Parsing problems will be published to internal logger and
	 * problemOutputStream.
	 * 
	 * @param problemOutputStream
	 */
	public SaxErrorHandler(final PrintStream problemOutputStream) {
		this.problemOutputStream = problemOutputStream;
	}

	public void warning(SAXParseException e) {
		final String msg = "[Warning]" + getLocationString(e);
		log.warn(msg);
		if (null != problemOutputStream) {
			problemOutputStream.print(msg);
		}
	}

	public void error(SAXParseException e) {
		final String msg = "[Error]" + getLocationString(e);
		log.error(msg);
		if (null != problemOutputStream) {
			problemOutputStream.print(msg);
		}
	}

	public void fatalError(SAXParseException e) throws SAXException {
		final String msg = "[Fatal Error]" + getLocationString(e);
		log.fatal(msg);
		if (null != problemOutputStream) {
			problemOutputStream.print(msg);
		}
	}

	private String getLocationString(SAXParseException e) {
		return " line " + e.getLineNumber() + ", column " + e.getColumnNumber()
				+ ":" + e.getMessage();
	}
}