package de.hdm_stuttgart.mi.sda2.ehcache;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import de.hdm_stuttgart.mi.sda2.ehcache.domain.Department;
import de.hdm_stuttgart.mi.sda2.ehcache.domain.Employee;

public class JpaTest {

   private static EntityManager manager1, manager2, manager3;

   /**
    * @param args
    */
   public static void main(String[] args) {


      EntityManagerFactory factory = Persistence.createEntityManagerFactory("persistenceUnit");
      manager1 = factory.createEntityManager();
      manager2 = factory.createEntityManager();
      manager3 = factory.createEntityManager();

      System.out.println("manager1 == manager2:" + (manager1 == manager2));


      try {
         createEmployees(manager1);
      } catch (Exception e) {
         e.printStackTrace();
      }

      System.out.println("Reading employees by manager 1");
      final Employee emp1[] = listEmployees(manager1);
      System.out.println("-------------------");
      
      findEmployees(manager2, emp1);
      findEmployees(manager3, emp1);
      
      displayCacheState();

      System.out.println("Reading employees by manager 3");
      listEmployees(manager3);
      System.out.println("-------------------");

      displayCacheState();
   }
   
   private static void findEmployees(final EntityManager manager, final Employee[] employees) {
      final EntityTransaction tx = manager.getTransaction();
      
      tx.begin();
      {
         for (final Employee e: employees) {
            System.out.println("Loading employee #" + e.getId() + " by manager " + manager);
            System.out.println(" ->" + manager.find(Employee.class, e.getId()));
         }
      }
      tx.commit();
   }

   private static void displayCacheState() {
      for (CacheManager cm : CacheManager.ALL_CACHE_MANAGERS) {
         System.out.println("----------- CacheManager:" + cm.getName());
         for (final String item: cm.getCacheNames()) {
            System.out.println("Cache:" + item);
            analyzeCache(cm, item);
            System.out.println("-------------------");
         }
      }
   }

   private static void analyzeCache(final CacheManager cm, final String cacheName) {
      Cache cache = cm.getCache(cacheName);

      System.out.println("isDisabled=" +cache.isDisabled());

      System.out.println("hit count:" + cache.getLiveCacheStatistics().getCacheHitCount());

      List<?> cachedObjects = cache.getKeys();
      System.out.println("Num of objects in cache:" + cachedObjects.size());
      for (final Object key: cachedObjects) {
         System.out.println("key:" + key);
      }
   }

   private static void createEmployees(final EntityManager manager) {
      final EntityTransaction tx = manager.getTransaction();
      tx.begin();
      {
         int numOfEmployees = manager.createQuery("Select a From Employee a", Employee.class).getResultList().size();
         if (numOfEmployees == 0) {
            Department department = new Department("java");
            manager.persist(department);

            manager.persist(new Employee("Jakab Gipsz",department));
            manager.persist(new Employee("Captain Nemo",department));
         }
      }
      tx.commit();
   }

   private static Employee[] listEmployees(final EntityManager manager) {
      final EntityTransaction tx = manager.getTransaction();
      
      List<Employee> resultList;
      tx.begin();
      {
         resultList = manager.createQuery("Select a From Employee a", Employee.class).
               setHint("org.hibernate.cacheable", true).getResultList();
         System.out.println("num of employess:" + resultList.size());
         for (Employee next : resultList) {
            manager.find(Employee.class, next.getId());
            System.out.println("next employee: " + next);
         }
      }
      tx.commit();
      return resultList.toArray(new Employee[0]);
   }
}
