package de.hdm_stuttgart.mi.sda2.usermanage.parse;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;


import de.hdm_stuttgart.mi.sda2.usermanage.rdbms.RdbmsHandler;

public class Parser {
   
   
   final RdbmsHandler rdbmsHandler;

   public Parser(final RdbmsHandler rdbmsHandler, final String groupFile, final String passwdFile)
         throws ParseError {
      this.rdbmsHandler = rdbmsHandler;
      
      // Reading /etc/group
      final BufferedReader groupStream = openFile(groupFile);
      String line;
      int lineNo = 1;
      try {
         lineNo = 1;
         while ( null != (line = groupStream.readLine())){
            rdbmsHandler.addGroup(new Group(line, lineNo++));
         }
      } catch (IOException e) {
         throw new ParseError("Unable to parse '" +
               groupFile+ "' file at line " + lineNo);
      }
      
      // Reading /etc/passwd
      final BufferedReader passwdStream = openFile(passwdFile);
      try {
          lineNo = 1;
          while ( null != (line = passwdStream.readLine())){
             rdbmsHandler.addUser(new User(line, lineNo++));
          }
          
          for (final String uid: Group.secondaryGroupsByUid.keySet()) {
        	  final User user = User.userByUid.get(uid);
        	  for (final Group group: Group.secondaryGroupsByUid.get(uid)) {
        		  rdbmsHandler.addUserGroup(user, group);
        	  }
          }
          
      } catch (IOException e) {
          throw new ParseError("Unable to parse '" +
                  passwdFile + "' file at line " + lineNo);
      }
      
      rdbmsHandler.commit();
   }
 
   
   BufferedReader openFile (final String filename) 
      throws ParseError {
      try {
         return new BufferedReader(new FileReader(filename));
      } catch (FileNotFoundException e) {
         throw new ParseError("Unable to open file '" + filename + "' for reading");
      }
   }
   
}
