package de.hdm_stuttgart.mi.sda2.account.sql;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sda2.account.Conf;
import de.hdm_stuttgart.mi.sda2.account.gui.ErrorReporter;

public class DbHandler {
	
	static private final Logger log = LogManager.getLogger(DbHandler.class);
	
	private final ErrorReporter errorReporter;
	public DbHandler(final ErrorReporter errorReporter) {
	   this.errorReporter = errorReporter;
	}
	
	Connection conn = null;
	boolean autoCommit;
	
	final static String updateAccountSql = "UPDATE Account SET balance = balance + ? WHERE number = ?";
	PreparedStatement updateAccount; 

	final static String accountSumSql = "SELECT SUM(balance) FROM Account";
	PreparedStatement accountSum; 

	private String getConnectionName() {
		return Conf.get("DbHandler.user") + '@' + Conf.get("DbHandler.jdbcUrl");
	}
	
	public boolean connect(final int isolationLevel, boolean autoCommit) {
		this.autoCommit = autoCommit;
		try {
			conn = DriverManager.getConnection(
					Conf.get("DbHandler.jdbcUrl"),
					Conf.get("DbHandler.user"),
					Conf.get("DbHandler.password"));
			conn.setAutoCommit(autoCommit);
			conn.setTransactionIsolation(isolationLevel);
			
			updateAccount = conn.prepareStatement(updateAccountSql);
			accountSum = conn.prepareStatement(accountSumSql);
			
			log.info("Connection '" + getConnectionName() + "' established" ); 
			return true;
		} catch (SQLException e) {
			log.info("Connection '" + getConnectionName() + "' failed:" + e.getMessage());
	        errorReporter.showExceptionAndExit("Connection '" + getConnectionName() + "' failed:", e, 1);
			return false;
		}
	}
	
	public void close() throws SQLException {
		if (null != conn) {
			conn.close();
			log.debug("Connection '" + getConnectionName() + "' closed" ); 
			conn = null;
		}
	}
	
	public void rollback() {
		try {
			conn.rollback();
			log.info("Rollback successfully executed");
		} catch (SQLException e) {
		   errorReporter.showExceptionAndExit("Rollback of transaction failed: ", e, 1);
		}
	}
	
	public void commit() {
		if (!autoCommit) {
			try {
				conn.commit();
				log.info("Changes successfully committed");
			} catch (SQLException e) {
			   errorReporter.showExceptionAndExit("Committing transaction failed: ", e, 1);
			}
		}
	}
	
	public int getAccountSum() {
		try {
			final ResultSet rs = accountSum.executeQuery();
			if (rs.next()) {
				final int ret = rs.getInt(1);
				rs.close();
				return  ret;
			} else {
			   errorReporter.showExceptionAndExit("Weird problem summing up balances problem: ", null, 1);
			}
		} catch (SQLException e) {
		   errorReporter.showExceptionAndExit("Committing transaction failed: ", e, 1);
		}
		return 0;
	}
	
	public void changeBy(int accountNo, int amount) {
		try {
			updateAccount.setInt(1, amount);
			updateAccount.setInt(2, accountNo);
			updateAccount.executeUpdate();
			log.info("Change account " + accountNo + " by " + amount);
		} catch (SQLException e) {
		   errorReporter.showExceptionAndExit("Update of account " + accountNo + " failed:", e, 1);
		}
	}
	
	
	
}
