<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xlink="http://www.w3.org/1999/xlink"
    xmlns:xila="http://www.w3.org/2001/XInclude/local-attributes"
    xmlns:xi="http://www.w3.org/2001/XInclude"
    xmlns:trans="http://docbook.org/ns/transclusion"
    xmlns:svg="http://www.w3.org/2000/svg"
    xmlns:m="http://www.w3.org/1998/Math/MathML"
    xmlns:html="http://www.w3.org/1999/xhtml"
    xmlns:db="http://docbook.org/ns/docbook"
    
    xmlns:s6hl="java:net.sf.xslthl.ConnectorSaxon6"
    xmlns:xslthl="http://xslthl.sf.net"
    extension-element-prefixes="s6hl xslthl"
    exclude-result-prefixes="xs xlink xila xi trans db svg html m"
    version="2.0">
    
    
    <xsl:template match="/db:book">
        <xsl:apply-templates select="db:chapter"/>
    </xsl:template>    
    
    <xsl:template match="db:chapter">
        <xsl:document href="{@xml:id}.html" method="html" indent="no">
            <html>
                <body >
                    <xsl:apply-templates/>
                </body>
            </html>
        </xsl:document>
    </xsl:template>
    
    <xsl:template match="db:title">
        <h1>
            <xsl:apply-templates/>
        </h1>
    </xsl:template>
    
    
    <!-- Title in ILIAS -->
    <xsl:template match="db:chapter/db:title"/>
    
    <!-- Sectioning -->
    <xsl:template match="db:section">
        <xsl:apply-templates/>
    </xsl:template>
    
    
    <!-- Blocks -->
    <xsl:template match="db:para">
        <p>
            <xsl:apply-templates select="*|text()" mode="link"/>
        </p>
    </xsl:template>
        
    <xsl:template match="db:screen">
        <pre style="width: fit-content; border-radius: 10px; padding: 5px; font-size:90%; font-family: 'Lucida Console', Monaco, monospace; background-color: #E7FFFE;">
            <xsl:apply-templates mode="link"/>
        </pre>
    </xsl:template>
    
    <!-- Lists -->
    
    <xsl:template match="db:itemizedlist">
        <ul>
            <xsl:apply-templates/>
        </ul>
    </xsl:template>
    
    <xsl:template match="db:orderedlist">
        <ol>
            <xsl:apply-templates select="@startingnumber"/>
            <xsl:apply-templates/>
        </ol>
    </xsl:template>
    
    <xsl:template match="@startingnumber">
        <xsl:attribute name="start">
            <xsl:value-of select="."/>
        </xsl:attribute>
    </xsl:template>
    
    <xsl:template match="db:listitem">
        <li>
            <xsl:apply-templates/>
        </li>
    </xsl:template>
    
    <!-- Admonitions -->
    
    <xsl:template match="db:tip">
        <div style="width: fit-content; border-radius: 10px; padding: 5px; border-style: solid; border-color: #CCCCCC; border-width: 1px; background-color: #DEFBDE;">
            <p>Hinweis:</p>
            <xsl:apply-templates/>
        </div>
    </xsl:template>
    
    <xsl:template match="db:caution">
        <div style="width: fit-content; border-radius: 10px; padding: 5px; border-style: solid; border-color: #CCCCCC; border-width: 1px;
                      background-color: #FFDFDF;">
            <p>Achtung:</p>
            <xsl:apply-templates/>
        </div>
    </xsl:template>
    <!-- Inlines -->
    
    <xsl:template match="db:xref">
        <span style="font-weight: bold;">
            <xsl:apply-templates select="id(@linkend)/db:title" mode="xref"/>
        </span>
    </xsl:template>
    
    
    <xsl:template match="db:informaltable">
        <table border="0">
            <xsl:for-each select="db:tr">
                <tr>
                    <xsl:for-each select="db:td">
                        <td valign="top">
                            <xsl:apply-templates select="text()|*"/>
                        </td>
                        
                    </xsl:for-each>
                </tr>
            </xsl:for-each>
            
        </table>
        
    </xsl:template>
    
    <xsl:template match="db:link|db:uri">
        <!-- link already present from template mode='link' -->
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="db:emphasis|db:acronym">
        <strong>
            <xsl:apply-templates/>
        </strong>
    </xsl:template>
    
    <xsl:template match="db:quote">
        <xsl:text>»</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>«</xsl:text>
    </xsl:template>
    
    <xsl:template match="db:productname|db:foreignphrase">
        <em>
            <xsl:apply-templates/>
        </em>
    </xsl:template>
    
    
    <xsl:template match="db:guimenu|db:guisubmenu">
        <em>
            <xsl:apply-templates/>
        </em>
    </xsl:template>
    
    <xsl:template match="db:filename">
        <span style="color: #810000;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
    
    <xsl:template match="db:package">
        <span style="font-size:90%; font-family: 'Lucida Console', Monaco, monospace; color: #FB6019;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>

    <xsl:template match="db:classname">
        <span style="font-size:90%; font-family: 'Lucida Console', Monaco, monospace;color: #FB9C19;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
    
    <xsl:template match="db:interfacename">
        <span style="font-size:90%; font-family: 'Lucida Console', Monaco, monospace;color: #B9B505;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
        
    <xsl:template match="db:methodname">
        <span style="font-size:90%; font-family: 'Lucida Console', Monaco, monospace;color: #C29B07;">
            <xsl:apply-templates/>
        </span>
    </xsl:template>
        
        
    <!-- link mode -->
    
    <xsl:template mode="link" match="db:package|db:classname|db:interfacename|db:methodname|db:link|db:uri|db:emphasis|db:code|db:filename|db:foreignphrase|db:productname">
        <xsl:choose>
            <xsl:when test="@xlink:href">
                <a href="{@xlink:href}" target="_blank">
                    <xsl:apply-templates select="."/>
                </a>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="."/>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <!-- Hack defeating the infamous ILIAS HTML input facility -->
    <xsl:template match="db:para/text()[0 = string-length(normalize-space(.))]" mode="link">
        <span>
            <xsl:text> </xsl:text>
        </span>
    </xsl:template>
    

    <!-- Ignoring most block elements with respect to linking -->
    <xsl:template match="*" mode="link">
        <xsl:apply-templates select="."/>
    </xsl:template>
    
    <!-- xref mode -->
    <xsl:template mode="xref" match="db:title">
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="*" mode="xref">
        <p style="background-color: red;">
            <xsl:text>No template defined for element '</xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text>' in mode 'xref'</xsl:text>
        </p>
    </xsl:template>

    <!-- The last resort -->
    
    <xsl:template match="*">
        <p style="background-color: red;">
            <xsl:text>No template defined for element '</xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text>'</xsl:text>
        </p>
    </xsl:template>
   
   <!-- 
   <xsl:template match="db:programlisting">
        <pre style="width: fit-content; border-radius: 10px; padding: 5px; font-size:90%; font-family: 'Lucida Console', Monaco, monospace; background-color: #F3ECFF;">
            <xsl:apply-templates select="text()|*" />
        </pre>
    </xsl:template>
   -->
    
    <xsl:template match="db:programlisting">
        <pre style="width: fit-content; border-radius: 10px; padding: 5px; font-size:90%; font-family: 'Lucida Console', Monaco, monospace; background-color: #F3ECFF;">
        <xsl:call-template name="syntax-highlight">
            <xsl:with-param name="language" select="@language" />
            <xsl:with-param name="source" select="." />
            <xsl:with-param name="config">file:///usr/share/hdm-docbook-xsl/docbook-xsl-1.79.2/highlighting/xslthl-config.xml</xsl:with-param>
        </xsl:call-template>
        </pre>
    </xsl:template>
    
    <xsl:template match="db:code">
        <code style='font-size:90%; color:black; font-family: "Lucida Console", Monaco, monospace; background-color: #F3ECFF;'>
            <xsl:call-template name="syntax-highlight">
                <xsl:with-param name="language" select="@language" />
                <xsl:with-param name="source" select="." />
                <xsl:with-param name="config">file:///usr/share/hdm-docbook-xsl/docbook-xsl-1.79.2/highlighting/xslthl-config.xml</xsl:with-param>
            </xsl:call-template>
        </code>
    </xsl:template>
    
    <xsl:template match="xslthl:keyword">
        <span style="font-weight: bold;color: #602060;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:string">
        <span style="color: #A00000;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:number|xslthl:hexnumber">
        <span style="color: #B08000;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:comment|xslthl:doccomment">
        <span style="font-style: italic;color: #808080;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:directive">
        <span style="color: #00A000;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:annotation">
        <span style="font-weight: bold;color: #808080;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:tag">
        <span style="font-weight: bold; color: #602060;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:attribute">
        <span style="color: #0050A0;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:value">
        <span style="color: #A00000;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template match="xslthl:doctype">
        <span style="color: #008080;">
            <xsl:value-of select="."/>
        </span>
    </xsl:template>
    
    <xsl:template name="syntax-highlight">
        <xsl:param name="language" />
        <xsl:param name="source" />
        <xsl:param name="config"></xsl:param>
        <xsl:choose>
            <xsl:when test="function-available('s6hl:highlight')">
                <xsl:variable name="highlighted" select="s6hl:highlight($language, $source, $config)" />
                <xsl:apply-templates select="$highlighted" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="$source/*|$source/text()" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>