package de.hdm_stuttgart.mi.bb;

import de.hdm_stuttgart.mi.bb.model.Topic;
import org.glassfish.jersey.client.ClientConfig;
import org.glassfish.jersey.jackson.JacksonFeature;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import javax.ws.rs.client.*;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import static java.util.ResourceBundle.getBundle;
import java.util.logging.Logger;
import java.util.logging.Level;

public class Id2Topic {

    static {
        final Logger mongoLogger = Logger.getLogger( "org.mongodb.driver" );
        mongoLogger.setLevel(Level.SEVERE); // e.g. or Log.WARNING, etc.
    }

    private static ResourceBundle config = getBundle("conf");

    static final
    ModifyNodebbTopic db = new ModifyNodebbTopic(
            config.getString("mongoUri"),
            config.getString("nodebbDb"));

    static final Client client = ClientBuilder.newClient(new ClientConfig().register(new JacksonFeature()));

    static final int LECTURE_CATEGORY = Integer.parseInt(config.getString("nodebbCategoryId"));
    static final String bearerToken = config.getString("nodebbBearerToken");

    static final String
            KEY_PAYLOAD = "payload",
            KEY_TOPICDATA = "topicData",
            KEY_POSTDATA = "postData";

    public static void main(String[] args) {


        final SAXBuilder saxBuilder = new SAXBuilder();

        if (0 == args.length) {
            System.err.println("Usage: id2topic <docbook2topic basefilename>");
        } else {
            final File inputFile = new File(args[0] + ".xml");

            try {
                final Document document = saxBuilder.build(inputFile);
                handleDocbok2idMap(document.getRootElement());

                final Format outFormat = Format.getPrettyFormat();

                // Serialize to console
                final XMLOutputter printer = new XMLOutputter(outFormat);

                final String outputFileName = args[0] + ".tid.xml";
                final FileWriter outputFile = new FileWriter(outputFileName);
                printer.output(document.getRootElement(), outputFile);
            } catch (JDOMException e) {
                System.err.println("Unable to parse " + inputFile);
            } catch (IOException e) {
                System.err.println("Unable to read " + inputFile + ":" +
                        e.getLocalizedMessage());
            }
        }
    }

    static void handleDocbok2idMap(final Element map) {
        String mapPrefix = map.getAttributeValue("prefix");

        for (final Element entry : map.getChildren("entry")) {
            final int tid = createTopic(mapPrefix + ":" + entry.getAttributeValue("id"),
                    entry.getChildText("title"),
                    entry.getChildText("content"));
            entry.setAttribute("tid", "" + tid);
        }
    }

    static int createTopic(String hdmId, String title, String content) {

        int tid = db.hdmIdExists(hdmId);
        if (tid < 0) { // No related topic
            final WebTarget topicsTarget = client.target(config.getString("nodebbWriteApiTopicsEndpoint"));

            final Topic topic = new Topic(LECTURE_CATEGORY, title, content);

            final Entity jsonTopic = Entity.json(topic);
            System.out.println("Feeding in Topic " + topic);
            final Map<String, Object> response = topicsTarget.request(MediaType.APPLICATION_JSON).
                    header(HttpHeaders.AUTHORIZATION, "Bearer " + bearerToken)
                    .accept(MediaType.TEXT_PLAIN_TYPE)
                    .post(jsonTopic, Map.class);

            //System.out.println("Result:" + response.get("code"));
            //printAttribs(response);

            final Map<String, Object> payload = (Map<String, Object>) response.get(KEY_PAYLOAD);
            final Map<String, Object> topicData = (Map<String, Object>) payload.get(KEY_TOPICDATA);

            int newTid = (int) topicData.get("tid");
            db.addHdmId(newTid, hdmId);
            return newTid;
        } else {
            System.out.println("hdmId '" + hdmId + "' already present, just adding tid");
            return tid;
            // Possibly aligning title and content with lecture notes?
            // Difficult: Incomplete NodeBB write API --> required MongoDB fiddling

            /*
            final WebTarget hdmTtopicTarget =
                    client.target(config.getString("nodebbReadApiTopicsEndpoint") + "/" + tid);

            final Response response = hdmTtopicTarget.request(MediaType.APPLICATION_JSON_TYPE).get();

            final Map<String, Object> hdmTopicMap = response.readEntity(Map.class);
            System.out.println("-----------------------Topic [tid=" + tid + "] data:");
            printAttribs(hdmTopicMap);
             */
        }
    }

    static void printAttribs(Map<String, Object> m) {
        for (String k : m.keySet()) {
            System.out.println("key '" + k + "', val '" + m.get(k) + "'");
        }
    }
}
