<?xml version="1.0" encoding="iso-8859-1"?>

<xsl:stylesheet 
  version="2.0"
  xmlns:fo="http://www.w3.org/1999/XSL/Format" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:kk="http://www.hdm-stuttgart.de/xmleditor/schema/kkabrechnung"
  >

  <xsl:include href="common.xsl"/>
  
  <xsl:output indent="yes" method="xml"/>
  
  <xsl:template match="/">
    <fo:root
      language="de"
      hyphenate="true"
      hyphenation-character="-"
      hyphenation-push-character-count="3"
      hyphenation-remain-character-count="3"
      font-family="Times">
      <!--____________________ Layout Master Set _____________________-->
      
      <fo:layout-master-set>
        <fo:simple-page-master master-name="pageLayout" 
          page-height="297mm" page-width="210mm"
          margin-top="20mm" margin-bottom="20mm"		
          margin-left="20mm" margin-right="20mm">
          <fo:region-body margin-top="20mm"/>
          <fo:region-before extent="20mm"/>
        </fo:simple-page-master>
      </fo:layout-master-set>
      
      <!--____________________ Page sequence content _________________-->
      
      <fo:page-sequence master-reference="pageLayout">
        <fo:static-content flow-name="xsl-region-before">
          <fo:block font-size="36pt" font-weight="bold">
          </fo:block>
        </fo:static-content>
        <fo:flow flow-name="xsl-region-body" font-size="12pt">
          <xsl:apply-templates select="kk:abrechnung"/>
        </fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>

  <xsl:template match="kk:abrechnung">
    <fo:block font-size="6pt" space-after="10mm">Landesamt fr Besoldung und Versorgung, BaW,<fo:inline font-weight="bold">70730</fo:inline>  Fellbach</fo:block>

    <xsl:for-each select="anschreiben/para">
      <fo:block space-after="3mm">
        <xsl:value-of select="."/>
      </fo:block>
    </xsl:for-each>

    <fo:block text-align="center" space-after="3mm" space-before="20mm">
      <xsl:value-of select="signature"/>
    </fo:block>

    <fo:block space-after="5mm">Personalnummer: 67297644/273G</fo:block>
    
    <xsl:apply-templates select="kk:unfaelle"/>

    <fo:block font-size="16pt" font-weight="bold">Verordnungen:</fo:block>

    <fo:table border-style="solid" border-width="0.5pt">
      <fo:table-column column-width="9mm"/>
      <fo:table-column column-width="20mm"/>
      <fo:table-column column-width="25mm"/>
      <fo:table-column column-width="33mm"/>
      <fo:table-column column-width="19mm"/>
      <fo:table-column column-width="19mm"/>
      <fo:table-column column-width="15mm"/>

      <fo:table-body border-width="0.5pt" border-style="solid">

        <fo:table-row font-size="8pt">
        <fo:table-cell number-rows-spanned="2" border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Nr</fo:block>
        </fo:table-cell>
        <fo:table-cell number-rows-spanned="2" border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Bezeichnung des Erkrankten</fo:block>
        </fo:table-cell>
        <fo:table-cell number-rows-spanned="2" border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Datum</fo:block>
        </fo:table-cell>
        <fo:table-cell number-rows-spanned="2" border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Art der Leistung</fo:block>
        </fo:table-cell>
        <fo:table-cell border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Betrag</fo:block>
        </fo:table-cell>
        <fo:table-cell border-right-width="0.5pt" border-right-style="solid">
          <fo:block margin="2mm">Kostenerstattung von anderer Seite</fo:block>
        </fo:table-cell>
        <fo:table-cell number-rows-spanned="2">
          <fo:block padding="2mm">Unfallbedingt?</fo:block>
        </fo:table-cell>
        </fo:table-row>
        <fo:table-row>
          <fo:table-cell border-right-width="0.5pt" border-right-style="solid">
            <fo:block>Euro</fo:block>
          </fo:table-cell>
          <fo:table-cell border-right-width="0.5pt" border-right-style="solid">
            <fo:block>Euro</fo:block>
          </fo:table-cell>
        </fo:table-row>


        <xsl:apply-templates select="kk:berechtigte"/>

        <fo:table-row>
          <fo:table-cell
            number-columns-spanned="4"
            border-top-width=".5pt" border-top-style="solid" 
            border-right-width="0.5pt" border-right-style="solid">
            <fo:block font-weight="bold" margin-right="1mm" text-align="end">Summe</fo:block>
          </fo:table-cell>
           


          <fo:table-cell
            border-top-width=".5pt" border-top-style="solid" 
            border-right-width="0.5pt" border-right-style="solid">
            <fo:block font-weight="bold" margin-right="1mm" text-align="end">
              <xsl:value-of select="format-number(sum(kk:verordnungen/kk:verordnung/@betrag),'.00')"/>
            </fo:block>
          </fo:table-cell>

          <fo:table-cell
            border-top-width=".5pt" border-top-style="solid" 
            border-right-width="0.5pt" border-right-style="solid">
            <fo:block font-weight="bold" margin-right="1mm" text-align="end">
              <xsl:value-of select="format-number(0.01 * sum(kk:verordnungen/kk:verordnung/(@betrag * (100 - key('berechtigterZuId', @ref)/@beihilfe))),'.00')"/>
            </fo:block>
          </fo:table-cell>

          <fo:table-cell
            border-top-width=".5pt" border-top-style="solid">
            <fo:block/>
          </fo:table-cell>

        </fo:table-row>
      </fo:table-body>
    </fo:table>

  </xsl:template>

  <xsl:template match="kk:berechtigte">
    <xsl:for-each select="*">
      <xsl:variable name="verord" select="key('verordnungByBerechtigter', @id)"></xsl:variable>
        <xsl:variable name="fremdErstattung" select="(100 - @beihilfe) div 100."/>
        <xsl:variable name="pTyp">
          <xsl:apply-templates select="." mode="personentyp"/>
        </xsl:variable>
        <xsl:for-each select="$verord">
          <fo:table-row>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>
              <xsl:value-of select="@nr"/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>
              <xsl:value-of select="$pTyp"/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>
              <xsl:value-of select="@datum"/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block>
              <xsl:call-template name="behandlungsart">
                <xsl:with-param name="typ" select="@typ"/>
              </xsl:call-template>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block text-align="end">
              <xsl:value-of select="@betrag"/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block text-align="end">
              <xsl:value-of select="format-number(@betrag * $fremdErstattung, '.00')"/>
            </fo:block>
          </fo:table-cell>
          <fo:table-cell border-top=".5pt solid black" border-right=".5pt solid black">
            <fo:block text-align="end">
              <xsl:choose>
                <xsl:when test="@unfall">
                  <xsl:text>Siehe </xsl:text>
                  <!-- Strange: id(@unfall) geht nicht! -->
                  <xsl:apply-templates select="key('unfallById', @unfall)" mode="nr" />
                </xsl:when>
                <xsl:otherwise>-</xsl:otherwise>
              </xsl:choose>
            </fo:block>
          </fo:table-cell>
          </fo:table-row>
          </xsl:for-each>
    </xsl:for-each>
  </xsl:template>
  

  <xsl:template match="kk:antragsteller" mode="personentyp">A</xsl:template>

  <xsl:template match="kk:ehepartner" mode="personentyp">E</xsl:template>

  <xsl:template match="kk:kind" mode="personentyp">
    <xsl:value-of select="kk:name"/>
  </xsl:template>

</xsl:stylesheet>

