<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
  
  <xsl:variable name="index" select="'index.html'"/>
  
  <xsl:output
    doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" 
    method="xml" indent="yes"/>
  
  <xsl:template match="/">
    <xsl:apply-templates select="book"/>
    <xsl:apply-templates select="book/chapter"/>
  </xsl:template>
  
  <xsl:template match="book">
    <xsl:result-document href="{$index}">
      <html>
        <head>
          <title><xsl:value-of select="title"/></title>
        </head>
        <body>
          <h1><xsl:value-of select="title"/></h1>
          <h2>Table of contents</h2>
          <ul>
            <xsl:for-each select="chapter">
              <li>
                <a href="{generate-id(.)}.html">
                  <xsl:value-of select="title"/>
                </a>
              </li>
            </xsl:for-each>
          </ul>
        </body>
      </html>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template match="chapter">
    <xsl:variable name="filebasename" select="generate-id(.)"/>
    <xsl:variable name="pageTitle">
      <xsl:text>Chapter </xsl:text>
      <xsl:value-of select="title"/>
    </xsl:variable>
    <xsl:result-document href="{$filebasename}.html">
      <html>
        <head>
          <title><xsl:value-of select="$pageTitle"/></title>
        </head>
        <body>
          <h1><xsl:value-of select="$pageTitle"/></h1>
          <xsl:apply-templates select="para"/>
        </body>
      </html>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template match="para">
    <p><xsl:apply-templates select="text()|*"/></p>
  </xsl:template>
  
  <xsl:template match="emph">
    <em><xsl:value-of select="."/></em>
  </xsl:template>
  
  <xsl:template match="externlink">
    <a href="{@href}"><xsl:value-of select="."/></a>
  </xsl:template>
  
  <xsl:template match="internlink">
    <!-- A little bit tricky, think about it! -->
    <a href="{generate-id(id(@ref))}.html"><xsl:value-of select="."/></a>
  </xsl:template>

</xsl:stylesheet>