<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0">
  
  <xsl:variable name="indexPage" select="'index.html'"/>

  <xsl:output
    doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" 
    method="xml" indent="yes"/>
  
  <xsl:template match="/">
    <xsl:apply-templates select="book"/>
    <xsl:apply-templates select="book/chapter"/>
  </xsl:template>
  
  <xsl:template match="book">
    <xsl:result-document href="{$indexPage}">
      <html>
        <head>
          <title><xsl:value-of select="title"/></title>
        </head>
        <body>
          <h1><xsl:value-of select="title"/></h1>
          <h2>Table of contents</h2>
          <ol>
            <xsl:for-each select="chapter">
              <li>
                <a href="{generate-id(.)}.html">
                  <xsl:value-of select="title"/>
                </a>
              </li>
            </xsl:for-each>
          </ol>
        </body>
      </html>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template match="chapter">
    <xsl:variable name="filebasename" select="generate-id(.)"/>
    <xsl:variable name="pageTitle">
      <xsl:text>Chapter </xsl:text>
      <xsl:apply-templates select="." mode="numbering"/>
      <xsl:text>:</xsl:text>
      <xsl:value-of select="title"/>
    </xsl:variable>
    <xsl:result-document href="{$filebasename}.html">
      <html>
        <head>
          <title><xsl:value-of select="$pageTitle"/></title>
        </head>
        <body>
          <h1><xsl:value-of select="$pageTitle"/></h1>
          <xsl:apply-templates select="para"/>
          <xsl:apply-templates select="." mode="navigation"/>
        </body>
      </html>
    </xsl:result-document>
  </xsl:template>
  
  <xsl:template match="chapter" mode="navigation">
    <table>
      <tr>
        <th>
          <a href="{$indexPage}">Table of contents</a>
        </th>
        <th>
          <xsl:choose>
            <xsl:when test="preceding-sibling::chapter">
              <xsl:for-each select="preceding-sibling::chapter[1]">
                <xsl:text>Previous:</xsl:text>
                <a href="{generate-id(.)}.html">
                  <xsl:value-of select="title"/>
                </a>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>First</xsl:otherwise>
          </xsl:choose>
        </th>
        <th>
          <xsl:choose>
            <xsl:when test="following-sibling::chapter">
              <xsl:for-each select="following-sibling::chapter[1]">
                <xsl:text>Next:</xsl:text>
                <a href="{generate-id(.)}.html">
                  <xsl:value-of select="title"/>
                </a>
              </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>Last</xsl:otherwise>
          </xsl:choose>
        </th>
      </tr>
    </table>
  </xsl:template>
  
  <xsl:template match="chapter" mode="numbering">
    <xsl:value-of select="1 + count(preceding-sibling::chapter)"/>
  </xsl:template>
  
  <xsl:template match="para">
    <p><xsl:apply-templates select="text()|*"/></p>
  </xsl:template>
  
  <xsl:template match="emph">
    <em><xsl:value-of select="."/></em>
  </xsl:template>
  
  <xsl:template match="externlink">
    <a href="{@href}"><xsl:value-of select="."/></a>
  </xsl:template>
  
  <xsl:template match="internlink">
    <!-- A little bit tricky, think about it! -->
    <a href="{generate-id(id(@ref))}.html"><xsl:value-of select="."/></a>
  </xsl:template>
  
</xsl:stylesheet>