package dom.xsl;

import java.io.IOException;
import java.util.List;


import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.jdom2.transform.XSLTransformException;
import org.jdom2.transform.XSLTransformer;
import org.jdom2.xpath.XPathFactory;

import dom.MySaxErrorHandler;

/**
 * Transforming an XML document to HTML by using
 * an XSL stylesheet.
 * @author goik
 *
 */
public class XmlRdbms2Html {
   private final SAXBuilder builder = new SAXBuilder();

   DbAccess db = new DbAccess();
   
   final XSLTransformer transformer;
   Document catalog;
   
   final org.jdom2.xpath.XPathExpression<Object> selectProducts = 
         XPathFactory.instance().compile("/catalog/product");

   /**
    * @param xslFilename the stylesheet being used for subsequent
    * transformations by {@link #transform(String, String)}.
    * 
    * @throws XSLTransformException XSL script may be ill-defined
    */
   public XmlRdbms2Html(final String xslFilename) throws XSLTransformException {
      builder.setErrorHandler(new MySaxErrorHandler(System.err));
      transformer =  new XSLTransformer(xslFilename);
   }
   
   /**
    * The actual workhorse carrying out the transformation
    * and adding prices from the database table.
    * 
    * @param xmlInFilename input file to be transformed
    * @param resultFilename the result file holding the generated HTML document
    * @throws JDOMException The transformation may fail for various reasons.
    * @throws IOException Access problem or invalid content
    */
   public void transform(final String xmlInFilename,
         final String resultFilename) throws JDOMException, IOException {

      catalog = builder.build(xmlInFilename);

      addPrices();

      final Document htmlResult = transformer.transform(catalog);

      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      printer.output(htmlResult, System.out);

   }
   private void addPrices() {
      final List<Object> products = selectProducts.evaluate(catalog.getRootElement());
      
      db.connect("jdbc:mysql://localhost:3306/hdm?serverTimezone=UTC", "hdmuser", "XYZ");
      for (Object p: products) {
         final Element product = (Element ) p;
         final String productId = product.getAttributeValue("id");
         product.setAttribute("price", db.readPrice(productId));
      }
      db.close();
   }
}
