package sax;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import sax.stat.v2.MyErrorHandler;
import sax.stat.v2.MyEventHandler;

/** A validating SAX parser */
public class Validate {
  
  /** 
   * @param argv Unused command arguments
   * @throws Exception File access problem or corrupt content.
   */
  public static void main(String argv[]) throws Exception{
    final Validate val = new Validate();
    val.parse("Input/Sax/catvalid.xml");
  }
  /** Set up validation
   * @throws SAXException Parsing may fail.
   * @throws ParserConfigurationException Unable to instantiate parser.
   */
  public Validate()
   throws SAXException, ParserConfigurationException{
      final SAXParserFactory saxPf = SAXParserFactory.newInstance();
      final SAXParser saxParser = saxPf.newSAXParser();
      xmlReader = saxParser.getXMLReader();
      xmlReader.setFeature(
          "http://xml.org/sax/features/validation", true);
      xmlReader.setContentHandler(eventHandler);
      xmlReader.setErrorHandler(errorHandler);
  }
  /** Initiate parsing
   * @param uri The resource to be parsed
   * @throws IOException Access related problems
   * @throws SAXException Parsing may fail
   */
  public void parse(final String uri) 
    throws IOException, SAXException{
    xmlReader.parse(uri);
  }
  /** @return number of elements in document, @see MyEventHandler */
  public int getElementCount() {
    return eventHandler.getElementCount();
  }
  private final XMLReader xmlReader;
  private final MyEventHandler eventHandler = new MyEventHandler();
  private final MyErrorHandler errorHandler = new MyErrorHandler();
}